@@ -1,8 +1,27 @@
+test_that("smap functions preserve names in output", {
+  core1 <- o_glycan_core_1()
+  core2 <- n_glycan_core()
+  structures <- c(core1, core2, core1)
+  names(structures) <- c("A", "B", "C")
+
+  # smap_int should preserve names
+  result <- smap_int(structures, igraph::vcount)
+  expect_equal(names(result), c("A", "B", "C"))
+
+  # smap_chr should preserve names
+  result_chr <- smap_chr(structures, ~ .x$anomer)
+  expect_equal(names(result_chr), c("A", "B", "C"))
+
+  # smap should preserve names in list
+  result_list <- smap(structures, ~ igraph::vcount(.x))
+  expect_equal(names(result_list), c("A", "B", "C"))
+})
+
 test_that("smap functions work with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test smap_int with regular function
   result <- smap_int(structures, igraph::vcount)
@@ -24,7 +43,7 @@ test_that("smap functions work with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1() 
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test smap_int with purrr lambda
   result_lambda <- smap_int(structures, ~ igraph::vcount(.x))
@@ -45,7 +64,7 @@ test_that("smap functions work with purrr-style lambda functions", {
 test_that("smap_unique works with purrr-style lambda functions", {
   # Create test structures with duplicates
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1, core1, core1)
+  structures <- c(core1, core1, core1)
   
   # Test with regular function
   result_regular <- smap_unique(structures, igraph::vcount)
@@ -59,7 +78,7 @@ test_that("smap_unique works with purrr-style lambda functions", {
 test_that("smap_structure works with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1, core1)
+  structures <- c(core1, core1)
   
   # Function that adds vertex names if not present
   add_names_regular <- function(g) {
@@ -104,7 +123,7 @@ test_that("ssome works with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()  # smaller structure
   core2 <- n_glycan_core()    # larger structure
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test if some structures have more than 5 vertices
   result <- ssome(structures, function(g) igraph::vcount(g) > 5)
@@ -125,7 +144,7 @@ test_that("ssome works with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test with purrr lambda
   result_lambda <- ssome(structures, ~ igraph::vcount(.x) > 5)
@@ -141,7 +160,7 @@ test_that("severy works with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test if all structures have at least 1 vertex (should be TRUE)
   result_true <- severy(structures, function(g) igraph::vcount(g) >= 1)
@@ -161,7 +180,7 @@ test_that("severy works with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test with purrr lambda
   result_lambda <- severy(structures, ~ igraph::vcount(.x) >= 1)
@@ -177,7 +196,7 @@ test_that("snone works with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test if no structures have more than 50 vertices (should be TRUE)
   result_true <- snone(structures, function(g) igraph::vcount(g) > 50)
@@ -197,7 +216,7 @@ test_that("snone works with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test with purrr lambda
   result_lambda <- snone(structures, ~ igraph::vcount(.x) > 50)
@@ -212,7 +231,7 @@ test_that("snone works with purrr-style lambda functions", {
 test_that("structure predicate functions work with duplicate structures efficiently", {
   # Create structures with many duplicates to test efficiency
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1, core1, core1, core1, core1)  # same structure 5 times
+  structures <- c(core1, core1, core1, core1, core1)  # same structure 5 times
   
   # These should only evaluate the predicate once for the unique structure
   result_some <- ssome(structures, function(g) igraph::vcount(g) > 3)
@@ -230,7 +249,7 @@ test_that("structure predicate functions work with duplicate structures efficien
 
 test_that("structure predicate functions handle edge cases", {
   # Test with single structure
-  single <- glycan_structure(o_glycan_core_1())
+  single <- o_glycan_core_1()
   expect_type(ssome(single, ~ igraph::vcount(.x) > 0), "logical")
   expect_type(severy(single, ~ igraph::vcount(.x) > 0), "logical")
   expect_type(snone(single, ~ igraph::vcount(.x) > 50), "logical")
@@ -242,7 +261,7 @@ test_that("smap2 functions work with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   weights <- c(1.0, 2.0, 1.5)
   
   # Test smap2_dbl with regular function
@@ -267,7 +286,7 @@ test_that("smap2 functions work with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   weights <- c(1.0, 2.0, 1.5)
   
   # Test smap2_dbl with purrr lambda
@@ -286,7 +305,7 @@ test_that("smap2 functions work with recycling", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test with single weight (should be recycled)
   single_weight <- 2.5
@@ -303,7 +322,7 @@ test_that("smap2_structure works with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)  # different structures
+  structures <- c(core1, core2)  # different structures
   
   # Function that adds a graph attribute (doesn't change structure topology)
   add_attribute <- function(g, attr_val) {
@@ -326,7 +345,7 @@ test_that("smap2_structure works with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)  # different structures
+  structures <- c(core1, core2)  # different structures
   
   # Test with purrr lambda that adds vertex attribute based on second argument
   labels <- c("label1", "label2")
@@ -346,7 +365,7 @@ test_that("smap2_structure works with purrr-style lambda functions", {
 test_that("smap2 functions handle duplicate structures efficiently", {
   # Create structures with duplicates to test efficiency
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1, core1, core1)  # same structure 3 times
+  structures <- c(core1, core1, core1)  # same structure 3 times
   weights <- c(1.0, 2.0, 1.0)  # first and third are same
   
   # This should only compute twice: once for (core1, 1.0) and once for (core1, 2.0)
@@ -363,7 +382,7 @@ test_that("smap2 functions handle duplicate structures efficiently", {
 
 test_that("smap2 functions validate inputs", {
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1)
+  structures <- core1
   
   # Test that first argument must be glycan_structure
   expect_error(smap2_dbl(c(1, 2, 3), c(1, 2, 3), ~ .x * .y), "glycan_structure")
@@ -378,7 +397,7 @@ test_that("smap2 functions validate inputs", {
 
 test_that("smap2 functions handle edge cases", {
   # Test with single structure
-  single <- glycan_structure(o_glycan_core_1())
+  single <- o_glycan_core_1()
   result <- smap2_dbl(single, 3.0, ~ igraph::vcount(.x) * .y)
   expect_equal(length(result), 1)
   expect_type(result, "double")
@@ -394,7 +413,7 @@ test_that("smap2 basic functionality works", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   values <- c("a", "b", "c")
   
   # Test smap2 (returns list)
@@ -413,7 +432,7 @@ test_that("smap2_chr works correctly", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   prefixes <- c("prefix1_", "prefix2_", "prefix3_")
   
   # Test smap2_chr
@@ -432,7 +451,7 @@ test_that("spmap functions work with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)
+  structures <- c(core1, core2)
   weights <- c(1.0, 2.0)
   factors <- c(2, 3)
   
@@ -461,7 +480,7 @@ test_that("spmap functions work with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)
+  structures <- c(core1, core2)
   weights <- c(1.0, 2.0)
   factors <- c(2, 3)
   
@@ -484,7 +503,7 @@ test_that("spmap functions work with recycling", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test with single weight (should be recycled)
   single_weight <- 2.0
@@ -503,26 +522,25 @@ test_that("spmap_structure works with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)
-  
+  structures <- c(core1, core2)
+
   # Function that adds graph attribute based on multiple arguments
   add_attributes <- function(g, label, value) {
     g <- igraph::set_graph_attr(g, "custom_label", label)
     g <- igraph::set_graph_attr(g, "custom_value", value)
     g
   }
-  
+
   labels <- c("label1", "label2")
   values <- c(10, 20)
   result <- spmap_structure(list(structures, labels, values), add_attributes)
-  
+
   expect_s3_class(result, "glyrepr_structure")
   expect_equal(length(result), 2)
-  
+
   # Check that attributes were added
-  structures_list <- attr(result, "structures")
-  data <- vctrs::vec_data(result)
-  first_structure <- structures_list[[vctrs::field(data, "iupac")[1]]]
+  graphs <- attr(result, "graphs")
+  first_structure <- graphs[[vctrs::vec_data(result)[1]]]
   expect_equal(igraph::graph_attr(first_structure, "custom_label"), "label1")
   expect_equal(igraph::graph_attr(first_structure, "custom_value"), 10)
 })
@@ -530,7 +548,7 @@ test_that("spmap_structure works with regular functions", {
 test_that("spmap functions handle duplicate structures efficiently", {
   # Create structures with duplicates to test efficiency
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1, core1, core1)
+  structures <- c(core1, core1, core1)
   weights <- c(1.0, 2.0, 1.0)
   factors <- c(2, 3, 2)
   
@@ -548,7 +566,7 @@ test_that("spmap functions handle duplicate structures efficiently", {
 
 test_that("spmap functions validate inputs", {
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1)
+  structures <- core1
   
   # Test that .l must be a list
   expect_error(spmap_dbl(structures, ~ igraph::vcount(.x)), "non-empty list")
@@ -567,7 +585,7 @@ test_that("spmap functions validate inputs", {
 
 test_that("spmap functions handle edge cases", {
   # Test with single structure
-  single <- glycan_structure(o_glycan_core_1())
+  single <- o_glycan_core_1()
   result <- spmap_dbl(list(single, 3.0, 2), ~ igraph::vcount(..1) * ..2 * ..3)
   expect_equal(length(result), 1)
   expect_type(result, "double")
@@ -583,7 +601,7 @@ test_that("spmap basic functionality works", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)
+  structures <- c(core1, core2)
   values1 <- c("a", "b")
   values2 <- c(1, 2)
   
@@ -606,7 +624,7 @@ test_that("simap functions work with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test simap_chr with regular function (using index)
   result <- simap_chr(structures, function(g, i) paste0("Structure_", i, "_vcount_", igraph::vcount(g)))
@@ -634,7 +652,7 @@ test_that("simap functions work with purrr-style lambda functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Test simap_chr with purrr lambda
   result_lambda <- simap_chr(structures, ~ paste0("Pos", .y, "_vertices", igraph::vcount(.x)))
@@ -651,7 +669,7 @@ test_that("simap_structure works with regular functions", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)
+  structures <- c(core1, core2)
   
   # Function that adds index/name as graph attribute
   add_index_attr <- function(g, idx) {
@@ -664,16 +682,15 @@ test_that("simap_structure works with regular functions", {
   expect_equal(length(result), 2)
   
   # Check that attributes were added
-  structures_list <- attr(result, "structures")
-  data <- vctrs::vec_data(result)
-  first_structure <- structures_list[[vctrs::field(data, "iupac")[1]]]
+  graphs <- attr(result, "graphs")
+  first_structure <- graphs[[vctrs::vec_data(result)[1]]]
   expect_equal(igraph::graph_attr(first_structure, "position"), 1)
 })
 
 test_that("simap functions handle duplicate structures efficiently", {
   # Create structures with duplicates to test efficiency
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1, core1, core1)
+  structures <- c(core1, core1, core1)
   
   # This should compute three times since indices are different: (core1, 1), (core1, 2), (core1, 3)
   result <- simap_chr(structures, function(g, i) paste0("Structure_", i, "_vcount_", igraph::vcount(g)))
@@ -694,7 +711,7 @@ test_that("simap functions handle duplicate structures efficiently", {
 
 test_that("simap functions validate inputs", {
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1)
+  structures <- core1
   
   # Test that first argument must be glycan_structure
   expect_error(simap_chr(c(1, 2, 3), ~ paste(.x, .y)), "glycan_structure")
@@ -709,7 +726,7 @@ test_that("simap functions validate inputs", {
 
 test_that("simap functions handle edge cases", {
   # Test with single structure
-  single <- glycan_structure(o_glycan_core_1())
+  single <- o_glycan_core_1()
   result <- simap_chr(single, ~ paste0("Index", .y, "_vertices", igraph::vcount(.x)))
   expect_equal(length(result), 1)
   expect_type(result, "character")
@@ -726,7 +743,7 @@ test_that("simap basic functionality works", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)
+  structures <- c(core1, core2)
   
   # Test simap (returns list)
   result <- simap(structures, function(g, i) {
@@ -747,7 +764,7 @@ test_that("smap parallel parameter works correctly", {
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2, core1)
+  structures <- c(core1, core2, core1)
   
   # Simple test function
   test_func <- function(g) igraph::vcount(g)
@@ -769,7 +786,7 @@ test_that("smap parallel parameter works correctly", {
 test_that("smap parallel parameter is not passed to user function", {
   # Create test structures
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1, core1)
+  structures <- c(core1, core1)
   
   # Function that would fail if .parallel is passed as argument
   strict_func <- function(g) {
@@ -789,29 +806,28 @@ test_that("smap handles additional arguments correctly with parallel parameter",
   # Create test structures
   core1 <- o_glycan_core_1()
   core2 <- n_glycan_core()
-  structures <- glycan_structure(core1, core2)
-  
+  structures <- c(core1, core2)
+
   # Function that takes additional arguments
   func_with_args <- function(g, multiplier = 1, offset = 0) {
     igraph::vcount(g) * multiplier + offset
   }
-  
+
   # Test with additional arguments and parallel parameter
   result1 <- smap_int(structures, func_with_args, multiplier = 2, offset = 1, .parallel = FALSE)
   result2 <- smap_int(structures, func_with_args, multiplier = 2, offset = 1, .parallel = NULL)
-  
+
   expect_equal(length(result1), 2)
   expect_equal(result1, result2)
   expect_type(result1, "integer")
-  
+
   # Verify the calculation is correct by computing expected values
   # Get the actual structures from the glycan_structure object
-  structures_list <- attr(structures, "structures")
-  data <- vctrs::vec_data(structures)
-  codes <- vctrs::field(data, "iupac")
-  
-  expected1 <- igraph::vcount(structures_list[[codes[1]]]) * 2 + 1
-  expected2 <- igraph::vcount(structures_list[[codes[2]]]) * 2 + 1
+  graphs <- attr(structures, "graphs")
+  codes <- vctrs::vec_data(structures)
+
+  expected1 <- igraph::vcount(graphs[[codes[1]]]) * 2 + 1
+  expected2 <- igraph::vcount(graphs[[codes[2]]]) * 2 + 1
   expect_equal(result1[1], expected1)
   expect_equal(result1[2], expected2)
 })
@@ -819,15 +835,15 @@ test_that("smap handles additional arguments correctly with parallel parameter",
 test_that("smap auto-parallel threshold behavior", {
   # Create a small dataset (should not trigger auto-parallel)
   core1 <- o_glycan_core_1()
-  small_structures <- glycan_structure(core1, core1, core1)  # 3 total, 1 unique
-  
+  small_structures <- c(core1, core1, core1)  # 3 total, 1 unique
+
   # Simple function
   simple_func <- function(g) igraph::vcount(g)
-  
+
   # With small dataset, auto-parallel should behave like sequential
   result_auto <- smap_int(small_structures, simple_func, .parallel = NULL)
   result_seq <- smap_int(small_structures, simple_func, .parallel = FALSE)
-  
+
   expect_equal(result_auto, result_seq)
   expect_equal(length(result_auto), 3)
   expect_type(result_auto, "integer")
@@ -836,13 +852,13 @@ test_that("smap auto-parallel threshold behavior", {
 test_that("smap parallel parameter validation", {
   # Create test structures
   core1 <- o_glycan_core_1()
-  structures <- glycan_structure(core1)
-  
+  structures <- core1
+
   # Test that .parallel accepts valid values
   expect_no_error(smap_int(structures, igraph::vcount, .parallel = TRUE))
   expect_no_error(smap_int(structures, igraph::vcount, .parallel = FALSE))
   expect_no_error(smap_int(structures, igraph::vcount, .parallel = NULL))
-  
+
   # Test that invalid .parallel values are handled gracefully
   # The function should either work or give a meaningful error, not crash
   expect_error(
@@ -857,13 +873,13 @@ test_that("smap_structure correctly updates unique structures count when modific
   structures <- as_glycan_structure(glycans)
 
   # Before modification: should have 2 unique structures
-  expect_equal(length(attr(structures, "structures")), 2)
+  expect_equal(length(attr(structures, "graphs")), 2)
 
   # Remove linkages - both structures should become identical
   result <- remove_linkages(structures)
 
   # After modification: should have only 1 unique structure
-  expect_equal(length(attr(result, "structures")), 1)
+  expect_equal(length(attr(result, "graphs")), 1)
 
   # Both elements should have the same IUPAC code
   expect_equal(as.character(result)[1], as.character(result)[2])
@@ -877,7 +893,7 @@ test_that("smap2_structure correctly updates unique structures count when modifi
   values <- c("test1", "test2")
 
   # Before modification: should have 2 unique structures
-  expect_equal(length(attr(structures, "structures")), 2)
+  expect_equal(length(attr(structures, "graphs")), 2)
 
   # Function that removes linkages regardless of second argument
   remove_linkages_func <- function(g, val) {
@@ -888,7 +904,7 @@ test_that("smap2_structure correctly updates unique structures count when modifi
   result <- smap2_structure(structures, values, remove_linkages_func)
 
   # After modification: should have only 1 unique structure
-  expect_equal(length(attr(result, "structures")), 1)
+  expect_equal(length(attr(result, "graphs")), 1)
 
   # Both elements should have the same IUPAC code
   expect_equal(as.character(result)[1], as.character(result)[2])
@@ -903,7 +919,7 @@ test_that("spmap_structure correctly updates unique structures count when modifi
   values2 <- c(1, 2)
 
   # Before modification: should have 2 unique structures
-  expect_equal(length(attr(structures, "structures")), 2)
+  expect_equal(length(attr(structures, "graphs")), 2)
 
   # Function that removes linkages regardless of other arguments
   remove_linkages_func <- function(g, val1, val2) {
@@ -914,7 +930,7 @@ test_that("spmap_structure correctly updates unique structures count when modifi
   result <- spmap_structure(list(structures, values1, values2), remove_linkages_func)
 
   # After modification: should have only 1 unique structure
-  expect_equal(length(attr(result, "structures")), 1)
+  expect_equal(length(attr(result, "graphs")), 1)
 
   # Both elements should have the same IUPAC code
   expect_equal(as.character(result)[1], as.character(result)[2])
@@ -1041,13 +1057,13 @@ test_that("smap2 refactored code maintains performance with complex inputs", {
 test_that("smap2 edge case: empty lists and mixed types", {
   glycan_codes <- c("Gal(a1-3)GalNAc(a1-", "GlcNAc(b1-2)Man(a1-3)[Man(a1-6)]Man(b1-4)GlcNAc(b1-4)GlcNAc(b1-")
   structures <- as_glycan_structure(glycan_codes)
-  
+
   # Test with empty lists and mixed simple/complex types
   mixed_data <- list(
     list(),              # Empty list
     "simple_string"      # Non-list data
   )
-  
+
   result <- smap2(structures, mixed_data, function(graph, data) {
     if (is.list(data)) {
       return(paste0("list_", length(data)))
@@ -1055,8 +1071,326 @@ test_that("smap2 edge case: empty lists and mixed types", {
       return(paste0("non_list_", data))
     }
   })
-  
+
   expect_length(result, 2)
   expect_equal(result[[1]], "list_0")
   expect_equal(result[[2]], "non_list_simple_string")
 })
+
+# Tests for names preservation -----------------------------------------
+
+test_that("smap_structure preserves names", {
+  core1 <- o_glycan_core_1()
+  structures <- c(core1, core1)
+  names(structures) <- c("X", "Y")
+
+  add_attr <- function(g) {
+    igraph::set_graph_attr(g, "test", "value")
+  }
+
+  result <- smap_structure(structures, add_attr)
+  expect_equal(names(result), c("X", "Y"))
+})
+
+test_that("smap2 functions preserve names", {
+  core1 <- o_glycan_core_1()
+  core2 <- n_glycan_core()
+  structures <- c(core1, core2, core1)
+  names(structures) <- c("A", "B", "C")
+  weights <- c(1.0, 2.0, 1.0)
+
+  # smap2_dbl should preserve names
+  result <- smap2_dbl(structures, weights, function(g, w) igraph::vcount(g) * w)
+  expect_equal(names(result), c("A", "B", "C"))
+
+  # smap2 should preserve names in list
+  result_list <- smap2(structures, weights, function(g, w) list(count = igraph::vcount(g), weight = w))
+  expect_equal(names(result_list), c("A", "B", "C"))
+})
+
+test_that("smap2_structure preserves names", {
+  core1 <- o_glycan_core_1()
+  structures <- c(core1, core1)
+  names(structures) <- c("X", "Y")
+  values <- c(1, 2)
+
+  add_attr <- function(g, v) {
+    igraph::set_graph_attr(g, "value", v)
+  }
+
+  result <- smap2_structure(structures, values, add_attr)
+  expect_equal(names(result), c("X", "Y"))
+})
+
+# Tests for spmap names preservation -----------------------------------------
+
+test_that("spmap functions preserve names", {
+  core1 <- o_glycan_core_1()
+  core2 <- n_glycan_core()
+  structures <- c(core1, core2)
+  names(structures) <- c("A", "B")
+  weights <- c(1.0, 2.0)
+  factors <- c(2, 3)
+
+  # spmap_dbl should preserve names
+  result <- spmap_dbl(list(structures, weights, factors),
+                      function(g, w, f) igraph::vcount(g) * w * f)
+  expect_equal(names(result), c("A", "B"))
+
+  # spmap should preserve names in list
+  result_list <- spmap(list(structures, weights, factors),
+                       function(g, w, f) list(count = igraph::vcount(g), weight = w))
+  expect_equal(names(result_list), c("A", "B"))
+})
+
+test_that("spmap_structure preserves names", {
+  core1 <- o_glycan_core_1()
+  structures <- c(core1, core1)
+  names(structures) <- c("X", "Y")
+  values1 <- c("a", "b")
+  values2 <- c(1, 2)
+
+  add_attrs <- function(g, v1, v2) {
+    g <- igraph::set_graph_attr(g, "label", v1)
+    igraph::set_graph_attr(g, "num", v2)
+  }
+
+  result <- spmap_structure(list(structures, values1, values2), add_attrs)
+  expect_equal(names(result), c("X", "Y"))
+})
+
+# Tests for simap names preservation -----------------------------------------
+
+test_that("simap functions preserve names", {
+  core1 <- o_glycan_core_1()
+  core2 <- n_glycan_core()
+  structures <- c(core1, core2, core1)
+  names(structures) <- c("A", "B", "C")
+
+  # simap_chr should preserve names
+  result <- simap_chr(structures, function(g, i) paste0("Structure_", i))
+  expect_equal(names(result), c("A", "B", "C"))
+
+  # simap should preserve names in list
+  result_list <- simap(structures, function(g, i) list(index = i))
+  expect_equal(names(result_list), c("A", "B", "C"))
+})
+
+test_that("simap_structure preserves names", {
+  core1 <- o_glycan_core_1()
+  structures <- c(core1, core1)
+  names(structures) <- c("X", "Y")
+
+  add_index_attr <- function(g, idx) {
+    igraph::set_graph_attr(g, "index", idx)
+  }
+
+  result <- simap_structure(structures, add_index_attr)
+  expect_equal(names(result), c("X", "Y"))
+})
+
+test_that("smap_unique documents behavior with named input", {
+  core1 <- o_glycan_core_1()
+  structures <- c(core1, core1, core1)
+  names(structures) <- c("A", "B", "C")  # All same structure, different names
+
+  # smap_unique operates on unique structures only
+  result <- smap_unique(structures, igraph::vcount)
+
+  # Result is named by structure hash (IUPAC), not input names
+  expect_true(is.null(names(result)) || startsWith(names(result), "Gal"))
+})
+
+test_that("get_structure_level preserves names", {
+  core1 <- o_glycan_core_1()
+  core2 <- n_glycan_core()
+  structures <- c(core1, core2, core1)
+  names(structures) <- c("A", "B", "C")
+
+  result <- get_structure_level(structures)
+  expect_equal(names(result), c("A", "B", "C"))
+})
+
+# Tests for NA handling in smap functions -----------------------------------
+
+test_that("smap skips NA elements", {
+  structs <- c(o_glycan_core_1(), glycan_structure(NA), n_glycan_core())
+  counts <- smap_int(structs, igraph::vcount)
+  expect_equal(length(counts), 3)
+  expect_false(is.na(counts[1]))
+  expect_true(is.na(counts[2]))
+  expect_false(is.na(counts[3]))
+})
+
+test_that("smap_structure skips NA elements", {
+  structs <- c(o_glycan_core_1(), glycan_structure(NA), n_glycan_core())
+  result <- smap_structure(structs, identity)
+  expect_equal(length(result), 3)
+  expect_false(is.na(result[1]))
+  expect_true(is.na(result[2]))
+  expect_false(is.na(result[3]))
+})
+
+test_that("ssome skips NA elements", {
+  structs <- c(glycan_structure(NA), o_glycan_core_1())
+  expect_true(ssome(structs, function(g) igraph::vcount(g) > 1))
+})
+
+test_that("severy skips NA elements", {
+  structs <- c(o_glycan_core_1(), glycan_structure(NA))
+  expect_true(severy(structs, function(g) igraph::vcount(g) > 1))
+})
+
+test_that("snone skips NA elements", {
+  structs <- glycan_structure(NA, NA)
+  expect_true(snone(structs, function(g) igraph::vcount(g) > 100))
+})
+
+test_that(".smap_base handles NA elements correctly", {
+  structs <- c(o_glycan_core_1(), glycan_structure(NA), n_glycan_core())
+
+  # smap_int should return NA for position 2
+  result <- smap_int(structs, igraph::vcount)
+  expect_equal(length(result), 3)
+  expect_false(is.na(result[1]))
+  expect_true(is.na(result[2]))
+  expect_false(is.na(result[3]))
+
+  # smap_chr should return NA for position 2
+  result_chr <- smap_chr(structs, ~ .x$anomer)
+  expect_equal(length(result_chr), 3)
+  expect_false(is.na(result_chr[1]))
+  expect_true(is.na(result_chr[2]))
+  expect_false(is.na(result_chr[3]))
+
+  # smap should return NULL for position 2 (for composition compatibility)
+  result_list <- smap(structs, igraph::vcount)
+  expect_equal(length(result_list), 3)
+  expect_false(is.na(result_list[[1]]))
+  expect_true(is.null(result_list[[2]]))
+  expect_false(is.na(result_list[[3]]))
+})
+
+test_that(".smap_base handles all-NA vector with correct NA types", {
+  # All-NA vector
+  all_na_structs <- glycan_structure(NA, NA, NA)
+
+  # smap_chr should return NA_character_, not "NA" string
+  result_chr <- smap_chr(all_na_structs, ~ .x$anomer)
+  expect_equal(length(result_chr), 3)
+  expect_type(result_chr, "character")
+  expect_true(all(is.na(result_chr)))
+  # Verify it's true NA_character_, not the string "NA"
+  expect_identical(result_chr[1], NA_character_)
+
+  # smap_int should return NA_integer_
+  result_int <- smap_int(all_na_structs, igraph::vcount)
+  expect_equal(length(result_int), 3)
+  expect_type(result_int, "integer")
+  expect_true(all(is.na(result_int)))
+
+  # smap_dbl should return NA_real_
+  result_dbl <- smap_dbl(all_na_structs, igraph::vcount)
+  expect_equal(length(result_dbl), 3)
+  expect_type(result_dbl, "double")
+  expect_true(all(is.na(result_dbl)))
+
+  # smap_lgl should return NA (logical)
+  result_lgl <- smap_lgl(all_na_structs, ~ igraph::vcount(.x) > 0)
+  expect_equal(length(result_lgl), 3)
+  expect_type(result_lgl, "logical")
+  expect_true(all(is.na(result_lgl)))
+
+  # smap returns list with NULL for NA elements
+  result_list <- smap(all_na_structs, igraph::vcount)
+  expect_equal(length(result_list), 3)
+  expect_true(all(purrr::map_lgl(result_list, is.null)))
+})
+
+# Tests for NA handling in smap2 functions -----------------------------------
+
+test_that("smap2 handles NA elements", {
+  structs <- c(o_glycan_core_1(), glycan_structure(NA), n_glycan_core())
+
+  # smap2_dbl should return NA for position 2
+  result <- smap2_dbl(structs, c(1, 2, 3), function(g, n) igraph::vcount(g) + n)
+  expect_equal(length(result), 3)
+  expect_false(is.na(result[1]))
+  expect_true(is.na(result[2]))
+  expect_false(is.na(result[3]))
+
+  # smap2 with lambda should work
+  result_lambda <- smap2_dbl(structs, c(1, 2, 3), ~ igraph::vcount(.x) + .y)
+  expect_equal(result, result_lambda)
+})
+
+test_that("smap2 handles NA at different positions", {
+  # NA at beginning
+  structs1 <- c(glycan_structure(NA), o_glycan_core_1(), n_glycan_core())
+  result1 <- smap2_dbl(structs1, c(1, 2, 3), ~ igraph::vcount(.x) + .y)
+  expect_true(is.na(result1[1]))
+  expect_false(is.na(result1[2]))
+  expect_false(is.na(result1[3]))
+
+  # NA at end
+  structs3 <- c(o_glycan_core_1(), n_glycan_core(), glycan_structure(NA))
+  result3 <- smap2_dbl(structs3, c(1, 2, 3), ~ igraph::vcount(.x) + .y)
+  expect_false(is.na(result3[1]))
+  expect_false(is.na(result3[2]))
+  expect_true(is.na(result3[3]))
+})
+
+test_that("smap2_structure handles NA elements", {
+  structs <- c(o_glycan_core_1(), glycan_structure(NA), n_glycan_core())
+  names(structs) <- c("A", "B", "C")
+  values <- c(1, 2, 3)
+
+  add_attr <- function(g, v) {
+    igraph::set_graph_attr(g, "value", v)
+  }
+
+  result <- smap2_structure(structs, values, add_attr)
+
+  expect_equal(length(result), 3)
+  expect_equal(names(result), c("A", "B", "C"))
+
+  # Check the graphs directly - there should be 2 unique graphs
+  result_graphs <- attr(result, "graphs")
+  expect_equal(length(result_graphs), 2)
+
+  # Check that attributes were set correctly on the graphs
+  expect_equal(igraph::graph_attr(result_graphs[[1]], "value"), 1)
+  expect_equal(igraph::graph_attr(result_graphs[[2]], "value"), 3)
+
+  # Verify result has correct codes (2 actual codes + 1 NA)
+  result_codes <- vctrs::vec_data(result)
+  expect_equal(sum(is.na(result_codes)), 1)
+  expect_equal(sum(!is.na(result_codes)), 2)
+})
+
+# Tests for NA handling in spmap functions -----------------------------------
+
+test_that("spmap handles NA elements", {
+  structs <- c(o_glycan_core_1(), glycan_structure(NA), n_glycan_core())
+
+  # spmap_dbl should return NA for position 2
+  result <- spmap_dbl(list(structs, c(1, 2, 3)), ~ igraph::vcount(..1) + ..2)
+  expect_equal(length(result), 3)
+  expect_false(is.na(result[1]))
+  expect_true(is.na(result[2]))
+  expect_false(is.na(result[3]))
+
+  # spmap with lambda should work
+  result_lambda <- spmap_dbl(list(structs, c(1, 2, 3)), function(g, n) igraph::vcount(g) + n)
+  expect_equal(result, result_lambda)
+})
+
+test_that("spmap_structure handles NA elements", {
+  structs <- c(o_glycan_core_1(), glycan_structure(NA), n_glycan_core())
+
+  result <- spmap_structure(list(structs, c(1, 2, 3)), function(g, n) g)
+  expect_equal(length(result), 3)
+  expect_false(is.na(result[1]))
+  expect_true(is.na(result[2]))
+  expect_false(is.na(result[3]))
+})
